# ALMA Data Reduction Script
# Tested in CASA version 5.1.1

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'Running fixplanets on fields with 0,0 coordinates',
              3: 'listobs',
              4: 'A priori flagging',
              5: 'Generation and time averaging of the WVR cal table',
              6: 'Generation of the Tsys cal table, antenna position, and delay tables',
              7: 'Application of the WVR, Tsys, antenna position, and delay cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^5.1.1', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 5.1.1')


# CALIBRATE_AMPLI: Titan
# CALIBRATE_ATMOSPHERE: 1224+213 Phase,3c273 - Bandpass,3c273 - Phase,M100,Titan
# CALIBRATE_BANDPASS: 3c273 - Bandpass
# CALIBRATE_FLUX: 
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: 1224+213 Phase,3c273 - Bandpass,3c273 - Phase,Titan
# CALIBRATE_POINTING: 3c273 - Pointing
# OBSERVE_TARGET: M100

# Using reference antenna = DV03 

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X273b43_X146.ms') == False:
    importasdm('uid___A002_X273b43_X146', asis='Antenna Station Receiver Source CalAtmosphere CalWVR', bdfflags=True)
  if applyonly != True: es.fixForCSV2555('uid___A002_X273b43_X146.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X273b43_X146.ms')

print "# A priori calibration"

# Running fixplanets on fields with 0,0 coordinates
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  fixplanets(vis = 'uid___A002_X273b43_X146.ms',
    field = '1', # Titan
    fixuvw = True)
  

# listobs
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.listobs')
  listobs(vis = 'uid___A002_X273b43_X146.ms',
    listfile = 'uid___A002_X273b43_X146.ms.listobs')
  
  

# A priori flagging
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X273b43_X146.ms',
    mode = 'unflag',
    flagbackup = False)
 
  flagdata(vis = 'uid___A002_X273b43_X146.ms',
    mode = 'shadow', 
    flagbackup = False)

  flagdata(vis = 'uid___A002_X273b43_X146.ms',
    mode = 'manual',
    spw = '1~16',
    autocorr = True,
    flagbackup = False)
  
  flagdata(vis = 'uid___A002_X273b43_X146.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = False)
  
  flagcmd(vis = 'uid___A002_X273b43_X146.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X273b43_X146.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X273b43_X146.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  
  flagmanager(vis = 'uid___A002_X273b43_X146.ms',
    mode = 'save',
    versionname = 'apriori')

# Generation and time averaging of the WVR cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.wvr') 
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X273b43_X146.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_X273b43_X146.ms',
    caltable = 'uid___A002_X273b43_X146.ms.wvr',
    toffset = -1,wvrflag='DV02',
    tie = ['M100,1224+213 Phase'],
    statsource = 'M100')
  
  casalog.setlogfile(mylogfile)
  
  # This is a temporary workaround, which will be included in a future version of CASA
  
  tb.open('uid___A002_X273b43_X146.ms.wvr', nomodify=False)
  count = 0
  numrows = tb.nrows()
  mycparamcol = tb.getcol('CPARAM')
  for i in range(0, numrows):
      if mycparamcol[0][0][i] == (1.+0.j):
          tb.putcell('FLAG', i, [[True]])
          count += 1
  tb.close()
  del mycparamcol
  if(numrows>0):
      print 'Flagged', count, 'of', numrows, 'solutions =', 100.*count/float(numrows),'%'
  
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.wvr.smooth') 
  
  smoothcal(vis = 'uid___A002_X273b43_X146.ms',
    tablein = 'uid___A002_X273b43_X146.ms.wvr',
    caltable = 'uid___A002_X273b43_X146.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 6.048)
  
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_X273b43_X146.ms.wvr.smooth', spw='1', antenna='DV11',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_X273b43_X146.ms.wvr.smooth.plots/uid___A002_X273b43_X146.ms.wvr.smooth') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.tsys') 
  gencal(vis = 'uid___A002_X273b43_X146.ms',
    caltable = 'uid___A002_X273b43_X146.ms.tsys',
    caltype = 'tsys')
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X273b43_X146.ms.tsys',
    mode = 'manual',
    spw = '9:0~3;124~127,11:0~3;124~127,13:0~3;124~127,15:0~3;124~127',
    flagbackup = False)
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X273b43_X146.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='5~123',showfdm=True, 
    field='', figfile='uid___A002_X273b43_X146.ms.tsys.plots.overlayTime/uid___A002_X273b43_X146.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.tsys', msName='uid___A002_X273b43_X146.ms', interactive=False) 


  os.system('rm -rf uid___A002_X273b43_X146.ms.antpos')
  gencal(vis = 'uid___A002_X273b43_X146.ms',
    caltable = 'uid___A002_X273b43_X146.ms.antpos',
    caltype = 'antpos',
    antenna = 'DV12,PM04',
    parameter=[-5.05331445738e-05,-0.000219840703066,-0.000512032681159,-0.000315962965482,0.000397399838991,-0.000581170175089])
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.delay')
  gaincal(vis = 'uid___A002_X273b43_X146.ms',
    caltable = 'uid___A002_X273b43_X146.ms.delay',
    field = '*Phase*', 
    spw = '1,3,5,7',
    solint = 'inf',
    combine = 'scan',
    refant = 'DV03',
    gaintable = 'uid___A002_X273b43_X146.ms.antpos',
    gaintype = 'K')           


# Application of the WVR and Tsys cal tables (and antpos)
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X273b43_X146.ms', tsystable = 'uid___A002_X273b43_X146.ms.tsys', tsysChanTol = 1)
  
  
  
  # Note: 3c273 - Pointing didn't have any Tsys measurement, and I couldn't find any close measurement. But this is not a science target, so this is probably Ok.

  applycal(vis = 'uid___A002_X273b43_X146.ms',
    field = '1',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X273b43_X146.ms.tsys', 'uid___A002_X273b43_X146.ms.wvr.smooth', 'uid___A002_X273b43_X146.ms.delay', 'uid___A002_X273b43_X146.ms.antpos'],
    gainfield = ['1', '', '', ''],
    interp = ['linear,linear', '', 'nearest', ''],
    spwmap = [tsysmap,[],[],[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_X273b43_X146.ms',
    field = '2',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X273b43_X146.ms.tsys', 'uid___A002_X273b43_X146.ms.wvr.smooth', 'uid___A002_X273b43_X146.ms.delay', 'uid___A002_X273b43_X146.ms.antpos'],
    gainfield = ['2', '', '', ''],
    interp = ['linear,linear', '', 'nearest', ''],
    spwmap = [tsysmap,[],[],[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_X273b43_X146.ms',
    field = '3',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X273b43_X146.ms.tsys', 'uid___A002_X273b43_X146.ms.wvr.smooth', 'uid___A002_X273b43_X146.ms.delay', 'uid___A002_X273b43_X146.ms.antpos'],
    gainfield = ['3', '', '', ''],
    interp = ['linear,linear', '', 'nearest', ''],
    spwmap = [tsysmap,[],[],[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_X273b43_X146.ms',
    field = '4',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X273b43_X146.ms.tsys', 'uid___A002_X273b43_X146.ms.wvr.smooth', 'uid___A002_X273b43_X146.ms.delay', 'uid___A002_X273b43_X146.ms.antpos'],
    gainfield = ['4', '', '', ''],
    interp = ['linear,linear', '', 'nearest', ''],
    spwmap = [tsysmap,[],[],[]],
    calwt = True,
    flagbackup = False)
  
  
  
  applycal(vis = 'uid___A002_X273b43_X146.ms',
    field = '5~49',
    spw = '1,3,5,7',
    gaintable = ['uid___A002_X273b43_X146.ms.tsys', 'uid___A002_X273b43_X146.ms.wvr.smooth', 'uid___A002_X273b43_X146.ms.delay', 'uid___A002_X273b43_X146.ms.antpos'],
    gainfield = ['5', '', '', ''],
    interp = ['linear,linear', '', 'nearest', ''],
    spwmap = [tsysmap,[],[],[]],
    calwt = True,
    flagbackup = False)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X273b43_X146.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.split.oldpos') 
  split(vis = 'uid___A002_X273b43_X146.ms',
    outputvis = 'uid___A002_X273b43_X146.ms.split.oldpos',
    datacolumn = 'corrected',
    spw = '1,3,5,7',
    keepflags = True)
  
  fixvis(vis='uid___A002_X273b43_X146.ms.split.oldpos', field='1224+213 Phase',
    phasecenter='J2000 12h24m54.458394 21d22m46.38858s',
    datacolumn='DATA',reuse=False,
    outputvis='uid___A002_X273b43_X146.ms.split')
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.split.listobs')
  listobs(vis = 'uid___A002_X273b43_X146.ms.split',
    listfile = 'uid___A002_X273b43_X146.ms.split.listobs')
  
  tb.open('uid___A002_X273b43_X146.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X273b43_X146.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X273b43_X146.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X273b43_X146.ms.split',
    mode = 'shadow',
    flagbackup = False)
  
  # Flagging atmospheric line(s)
  
  flagdata(vis = 'uid___A002_X273b43_X146.ms.split',
    mode = 'manual',
    spw = '0:2465~3839',
    field = '1',
    flagbackup = False)

  # Flag channels at edges
  
  flagdata(vis='uid___A002_X273b43_X146.ms.split',
    field='',
    mode='manual',
    spw='0~3:0~80;3760~3839',
    flagbackup=False)

  # Flag channels that are off
  
  flagdata(vis='uid___A002_X273b43_X146.ms.split',
    field='',
    mode='manual',
    spw='0~3:239;447~448;720~721;2847~2848',
    flagbackup=False)

  # Flag XX for DV12, spw3, which is dead
  
  flagdata(vis='uid___A002_X273b43_X146.ms.split',field='',mode='manual',antenna='DV12',correlation='XX',spw='3',flagbackup=False)

  # Flag Titan short baselines (< 40m; increased scatter in uv dist plot, probably too close to Saturn) and the CO emission feature in Titan's atmosphere @115.3GHz
        
  flagdata(vis='uid___A002_X273b43_X146.ms.split',field='Titan',mode='manual',uvrange='0~40',flagbackup = False)
  flagdata(vis='uid___A002_X273b43_X146.ms.split',field='Titan',mode='manual',spw='0:1201~3839',flagbackup = False)

  # Flagging spectral feature found at ~101.7 GHz in spw3 that could not be calibrated out
        
  flagdata(vis='uid___A002_X273b43_X146.ms.split',field='M100',mode='manual',spw='3:0~400',flagbackup=False)
  

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_X273b43_X146.ms.split',
    field = '1', # Titan
    spw = '0,1,2,3',
    standard = 'Butler-JPL-Horizons 2012')
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.setjy.field*.png') 
  for i in ['1']:
    plotms(vis = 'uid___A002_X273b43_X146.ms.split',
      xaxis = 'uvdist',
      yaxis = 'amp',
      ydatacolumn = 'model',
      field = str(i),
      spw = '0,1,2,3',
      avgchannel = '9999',
      coloraxis = 'spw',
      plotfile = 'uid___A002_X273b43_X146.ms.split.setjy.field'+i+'.png')
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X273b43_X146.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.ap_pre_bandpass',
    field = '2', # 3c273 - Bandpass
    spw = '0:1536~2304,1:1536~2304,2:1536~2304,3:1536~2304',
    solint = 'int',
    refant = 'DV03',
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.split.ap_pre_bandpass', msName='uid___A002_X273b43_X146.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.bandpass',
    field = '2', # 3c273 - Bandpass
    solint = 'inf',
    combine = 'scan',
    refant = 'DV03',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_X273b43_X146.ms.split.ap_pre_bandpass')
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch') 
  
  bandpass(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch',
    field = '2', # 3c273 - Bandpass
    solint = 'inf,20ch',
    combine = 'scan',
    refant = 'DV03',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_X273b43_X146.ms.split.ap_pre_bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch', msName='uid___A002_X273b43_X146.ms.split', interactive=False) 
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.split.bandpass', msName='uid___A002_X273b43_X146.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X273b43_X146.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.split.phase_int') 
  
  gaincal(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.phase_int',
    field = '1~4', # Titan,3c273 - Bandpass,1224+213 Phase,3c273 - Phase
    solint = 'int',
    refant = 'DV03',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.split.phase_int', msName='uid___A002_X273b43_X146.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.ampli_inf',
    field = '1~4', # Titan,3c273 - Bandpass,1224+213 Phase,3c273 - Phase
    solint = 'inf',
    refant = 'DV03',
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch', 'uid___A002_X273b43_X146.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.split.ampli_inf', msName='uid___A002_X273b43_X146.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X273b43_X146.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X273b43_X146.ms.split.flux_inf',
    reference = '1', # Titan
    transfer = '*Phase*,*Band*',
    refspwmap = [1,1,2,3])
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_X273b43_X146.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_X273b43_X146.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_X273b43_X146.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X273b43_X146.ms.split',
    caltable = 'uid___A002_X273b43_X146.ms.split.phase_inf',
    field = '1~4', # Titan,3c273 - Bandpass,1224+213 Phase,3c273 - Phase
    solint = 'inf',
    refant = 'DV03',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch')
  
  if applyonly != True: es.checkCalTable('uid___A002_X273b43_X146.ms.split.phase_inf', msName='uid___A002_X273b43_X146.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X273b43_X146.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['1', '2']: # Titan,3c273 - Bandpass
    applycal(vis = 'uid___A002_X273b43_X146.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch', 'uid___A002_X273b43_X146.ms.split.phase_int', 'uid___A002_X273b43_X146.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = True,
      flagbackup = False)

  applycal(vis = 'uid___A002_X273b43_X146.ms.split',
    field = '4', # 3c273 - Phase
    gaintable = ['uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch', 'uid___A002_X273b43_X146.ms.split.phase_inf', 'uid___A002_X273b43_X146.ms.split.flux_inf'],
    gainfield = ['', '3', '3'],
    interp = 'linear,linear',
    calwt = True,
    flagbackup = False)

  applycal(vis = 'uid___A002_X273b43_X146.ms.split',
    field = '3,6~49', # M100
    gaintable = ['uid___A002_X273b43_X146.ms.split.bandpass_smooth20ch', 'uid___A002_X273b43_X146.ms.split.phase_inf', 'uid___A002_X273b43_X146.ms.split.flux_inf'],
    gainfield = ['', '3', '3'], # 1224+213 Phase
    interp = 'linear,linear',
    calwt = True,
    flagbackup = False)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X273b43_X146.ms.split.cal') 
  split(vis = 'uid___A002_X273b43_X146.ms.split',
    outputvis = 'uid___A002_X273b43_X146.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = True)
  
  

